var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { jsx as _jsx } from "react/jsx-runtime";
import { Utils } from "liqvid";
import { useCallback, useEffect, useMemo } from "react";
const { onClick } = Utils.mobile;
// import {onClick} from "@liqvid/utils/react";
import { useBoothStore } from "../store";
import { keymap } from "@codemirror/view";
import { shortcuts } from "../extensions";
const mac = navigator.platform === "MacIntel";
/** Div to hold buttons. */
export const Buttons = (props) => {
    return (_jsx("div", Object.assign({ className: "lqv-cb-buttons" }, { children: props.children })));
};
/** Button for clearing the output/console. */
export function Clear() {
    const useStore = useBoothStore();
    const clear = useCallback(() => {
        useStore.setState({ messages: [] });
    }, []);
    const events = useMemo(() => onClick(clear), []);
    /* add keyboard shortcuts */
    useEffect(() => {
        for (const view of getAllEditors(useStore.getState())) {
            addKeyboardShortcuts(view, "Mod-L", () => {
                clear();
                return true;
            });
        }
    }, []);
    const label = "Clear";
    return (_jsx("button", Object.assign({ className: "lqv-cb-clear" }, events, { "aria-label": label, title: label }, { children: "Clear" })));
}
/** Button for copying the contents of one buffer to another. */
export const Copy = (props) => {
    const useStore = useBoothStore();
    const copy = useCallback(() => {
        const { groups } = useStore.getState();
        for (const key in groups) {
            // look for source documents
            if (!key.startsWith(props.from + ":"))
                continue;
            const filename = key.slice((props.from + ":").length);
            const source = groups[key];
            // find corresponding target document
            const target = groups[props.to + ":" + filename];
            if (!target) {
                continue;
            }
            // copy document
            target.dispatch(target.state.update({
                changes: {
                    from: 0,
                    to: target.state.doc.length,
                    insert: source.state.doc
                }
            }));
        }
    }, []);
    const events = useMemo(() => onClick(copy), []);
    return (_jsx("button", Object.assign({ className: "lqv-cb-copy" }, events, { children: "Copy" })));
};
/** Button for resetting editor. */
export const Reset = (props) => {
    const useStore = useBoothStore();
    const events = useMemo(() => onClick(e => {
        var _a;
        const { groups } = useStore.getState();
        for (const groupName in props.to) {
            const source = props.to[groupName];
            const target = groups[groupName];
            for (const file of source.files) {
                const view = (_a = target.files.find(_ => _.filename === file.filename)) === null || _a === void 0 ? void 0 : _a.view;
                if (!view) {
                    throw new Error(`Could not find ${file.filename} in group ${groupName}`);
                    continue;
                }
                view.dispatch(view.state.update({
                    changes: {
                        from: 0,
                        to: file.content.length,
                        insert: file.content
                    }
                }));
            }
        }
    }), []);
    return (_jsx("button", Object.assign({ className: "lqv-cb-reset" }, events, { children: "Reset" })));
};
/** Button for running the code. */
export function Run() {
    const useStore = useBoothStore();
    // run callback
    const run = useCallback(() => {
        useStore.setState(prev => ({ run: prev.run + 1 }));
    }, []);
    /* add keyboard shortcuts */
    useEffect(() => {
        for (const view of getAllEditors(useStore.getState())) {
            addKeyboardShortcuts(view, "Mod-Enter", () => {
                run();
                return true;
            });
        }
    }, []);
    // click events
    const events = useMemo(() => onClick(run), []);
    return (_jsx("button", Object.assign({ className: "lqv-cb-run" }, events, { children: "Run" })));
}
/** Group selection tab. */
export const Tab = (props) => {
    const { children, id } = props, attrs = __rest(props, ["children", "id"]);
    const useStore = useBoothStore();
    const active = useStore(state => state.activeGroup === id);
    const events = useMemo(() => onClick(() => {
        useStore.setState({ activeGroup: id });
    }), []);
    return (_jsx("button", Object.assign({ "aria-controls": `panel-${id}`, id: `tab-${id}`, "aria-selected": active, role: "tab" }, events, attrs, { children: props.children })));
};
/** Holds a list of tabs. */
export const TabList = (props) => {
    const { children } = props, attrs = __rest(props, ["children"]);
    return (_jsx("div", Object.assign({ role: "tablist" }, attrs, { children: children })));
};
/** Add keyboard shortcuts to editors */
function addKeyboardShortcuts(view, key, run) {
    view.dispatch({
        effects: shortcuts.reconfigure([
            shortcuts.get(view.state),
            keymap.of([
                { key, run }
            ])
        ])
    });
}
function getAllEditors(state) {
    const editors = [];
    for (const key in state.groups) {
        for (const file of state.groups[key].files) {
            editors.push(file.view);
        }
    }
    return editors;
}
/** Format key sequences with special characters on Mac */
function fmtSeq(str) {
    if (navigator.platform !== "MacIntel")
        return str;
    if (str === void 0)
        return str;
    return str.split("+").map(k => {
        if (k === "Ctrl")
            return "^";
        else if (k === "Alt")
            return "⌥";
        if (k === "Shift")
            return "⇧";
        if (k === "Meta")
            return "⌘";
        return k;
    }).join("");
}
