import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { html } from "@codemirror/lang-html";
import { css } from "@codemirror/lang-css";
import { javascript } from "@codemirror/lang-javascript";
import { CodeBooth } from "..";
import { Buttons, Clear, Copy, Reset, Run, Tab, TabList } from "../components/buttons";
import { TabPanel } from "../components/TabPanel";
import { FileTabs } from "../components/FileTabs";
import { Console } from "../components/Console";
import { Editor } from "../components/Editor";
import { Replay } from "../components/Replay";
import { Resize } from "../components/Resize";
import { Preview } from "../components/Preview";
import { basicSetup } from "../extensions";
/** HTML run function */
const run = (files, setState) => {
};
/** Replay handling function */
const handle = (useStore, key, doc) => {
    switch (key) {
        case "Ctrl-Enter":
        case "Cmd-Enter":
            const { getActiveFiles, run } = useStore.getState();
            // if we're replaying, latest changes haven't been dispatched to editor yet
            const files = getActiveFiles();
            files["untitled.py"] = doc.toString();
            run(files, useStore.setState);
            break;
        case "Ctrl-L":
        case "Cmd-K":
            useStore.setState({ messages: [] });
            break;
    }
};
/** HTML demo */
export function HTMLDemo(props) {
    const initial = props.initial || {
        demo: {
            activeFile: "index.html",
            files: [
                { content: "", filename: "index.html" },
                { content: "", filename: "script.js" },
                { content: "", filename: "style.css" }
            ]
        }
    };
    return (_jsxs(CodeBooth, Object.assign({ run: run }, { children: [_jsx(FileTabs, {}), Object.keys(initial).map(group => _jsx(TabPanel, Object.assign({ id: group }, { children: initial[group].files.map(file => _jsx(Editor, { extensions: [basicSetup, extensionFromFilename(file.filename)], filename: file.filename }, file.filename)) }), group)), _jsx(Resize, {}), _jsx(Console, {}), _jsxs(Buttons, { children: [_jsx(Reset, { to: initial }), _jsx(Run, {}), _jsx(Clear, {})] })] })));
}
/** Interactive HTML replay */
export const HTMLReplay = (props) => {
    return (_jsxs(CodeBooth, Object.assign({ activeGroup: "replay", run: run }, { children: [_jsx(TabPanel, Object.assign({ id: "replay" }, { children: _jsx(Replay, { extensions: [basicSetup, html()], filename: "untitled.py", handle: handle, replay: props.replay, start: props.start }) })), _jsx(TabPanel, Object.assign({ id: "playground" }, { children: _jsx(Editor, { extensions: [basicSetup, html()], filename: "untitled.py" }) })), _jsx(Resize, {}), _jsx(Console, {}), _jsxs(Buttons, { children: [_jsxs(TabList, { children: [_jsx(Tab, Object.assign({ id: "replay" }, { children: "Code" })), _jsx(Tab, Object.assign({ id: "playground" }, { children: "Playground" }))] }), _jsx(Copy, { from: "replay", to: "playground" }), _jsx(Run, {}), _jsx(Clear, {})] }), props.children] })));
};
/** Record HTML demos */
export const HTMLRecord = () => {
    return (_jsxs(CodeBooth, { children: [_jsx(FileTabs, {}), _jsxs(TabPanel, Object.assign({ id: "record" }, { children: [_jsx(Editor, { extensions: [basicSetup, html()], filename: "index.html" }), _jsx(Editor, { extensions: [basicSetup, css()], filename: "style.css" }), _jsx(Editor, { extensions: [basicSetup, javascript()], filename: "script.js" })] })), _jsx(Resize, {}), _jsx(Buttons, { children: _jsx(Run, {}) }), _jsx(Preview, {})] }));
};
// /* messaging */
// const messages = useStore(state => state.messages);
// useEffect(() => {
//   function update(msg: MessageEvent) {
//     if (msg.data.type === "console.log") {
//       useStore.setState(prev => ({
//         ...prev,
//         messages: prev.messages.concat(msg.data.content)
//       }));
//     } else if (msg.data.type === "console.clear") {
//       useStore.setState(prev => ({
//         ...prev,
//         messages: []
//       }))
//     }
//   }
//   window.addEventListener("message", update);
//   return () => {
//     window.removeEventListener("message", update);
//   }
// }, []);
/** Get CM extension appropriate to filename extension */
function extensionFromFilename(filename) {
    switch (true) {
        case filename.endsWith(".css"):
            return css();
        case filename.endsWith(".html"):
            return html();
        case filename.endsWith(".js"):
            return javascript();
    }
}
