import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { classHighlightStyle } from "@codemirror/highlight";
import { python } from "@codemirror/lang-python";
import { CodeBooth } from "..";
import { Buttons, Clear, Copy, Reset, Run, Tab, TabList } from "../components/buttons";
import { Console } from "../components/Console";
import { Editor } from "../components/Editor";
import { Replay } from "../components/Replay";
import { Resize } from "../components/Resize";
import { TabPanel } from "../components/TabPanel";
import { basicSetup } from "../extensions";
import { PythonInterpreter } from "../interpreters/skulpt";
const interpreter = new PythonInterpreter();
/** Python run function */
const run = (files, setState) => {
    const file = files["untitled.py"];
    let messages;
    try {
        const output = interpreter.runSync(file);
        messages = output.filter(_ => _.trim()).map(text => ({ classNames: ["user"], text }));
    }
    catch (e) {
        messages = [{
                classNames: ["user", "error"],
                text: e.args.v[0].v + "\n"
            }];
    }
    setState(prev => ({ messages: prev.messages.concat(messages) }));
};
/** Replay handling function */
const handle = (useStore, key, doc) => {
    switch (key) {
        case "Ctrl-Enter":
        case "Cmd-Enter":
            const { getActiveFiles, run } = useStore.getState();
            // if we're replaying, latest changes haven't been dispatched to editor yet
            const files = getActiveFiles();
            files["untitled.py"] = doc.toString();
            run(files, useStore.setState);
            break;
        case "Cmd-K":
            useStore.setState({ messages: [] });
            break;
    }
};
/** Python demo */
export function PythonDemo(props) {
    var _a;
    return (_jsxs(CodeBooth, Object.assign({ run: run }, { children: [_jsx(Editor, { extensions: [basicSetup, classHighlightStyle, python(), ...((_a = props.extensions) !== null && _a !== void 0 ? _a : [])], filename: "untitled.py" }), _jsx(Resize, {}), _jsx(Console, {}), _jsxs(Buttons, { children: [_jsx(Reset, {}), _jsx(Run, {}), _jsx(Clear, {})] })] })));
}
/** Interactive Python replay */
export const PythonReplay = (props) => {
    return (_jsxs(CodeBooth, Object.assign({ activeGroup: "replay", run: run }, { children: [_jsx(TabPanel, Object.assign({ id: "replay" }, { children: _jsx(Replay, { extensions: [basicSetup, classHighlightStyle, python()], filename: "untitled.py", handle: handle, replay: props.replay, start: props.start }) })), _jsx(TabPanel, Object.assign({ id: "playground" }, { children: _jsx(Editor, { extensions: [basicSetup, classHighlightStyle, python()], filename: "untitled.py" }) })), _jsx(Resize, {}), _jsx(Console, {}), _jsxs(Buttons, { children: [_jsxs(TabList, { children: [_jsx(Tab, Object.assign({ id: "replay" }, { children: "Code" })), _jsx(Tab, Object.assign({ id: "playground" }, { children: "Playground" }))] }), _jsx(Copy, { from: "replay", to: "playground" }), _jsx(Run, {}), _jsx(Clear, {})] }), props.children] })));
};
